/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.convert;

import org.bioimageanalysis.icy.extension.plugin.abstract_.Plugin;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.bioimageanalysis.icy.model.OMEUtil;
import org.bioimageanalysis.icy.model.image.IcyBufferedImageUtil;
import org.bioimageanalysis.icy.model.lut.LUT;
import org.bioimageanalysis.icy.model.sequence.Sequence;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;
import plugins.adufour.blocks.tools.sequence.SequenceBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.Var;
import plugins.adufour.vars.lang.VarEnum;
import plugins.adufour.vars.lang.VarSequence;
import plugins.adufour.vars.util.VarException;

import java.awt.image.BufferedImage;

/**
 * Block to render a given Sequence using a specific LUT.<br>
 * The result can be provided ARGB, RGB or GRAY sequence.
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("Convert Color")
@IcyPluginIcon(path = "/SequenceBlocks_icon.png")
public class ConvertColor extends Plugin implements SequenceBlock {
    public enum ColorConversion {
        GRAY, RGB, ARGB
    }

    final protected VarSequence EZseq = new VarSequence("Sequence", null);
    final protected VarEnum<ColorConversion> EZtype = new VarEnum<>("Conversion", ColorConversion.ARGB);
    final protected Var<LUT> EZlut = new Var<>("Lut", LUT.class);
    final protected VarSequence varOut = new VarSequence("Out", null);

    @Override
    public void run() {
        final Sequence s;

        // EZ PLUG
        s = EZseq.getValue();
        if (s == null)
            throw new VarException(EZseq, "Input sequence is null.");

        try {
            varOut.setValue(convertColor(s, getImageType(EZtype.getValue()), EZlut.getValue()));
        }
        catch (final InterruptedException e) {
            // nothing to do
        }
    }

    @Override
    public void declareInput(final @NotNull VarList inputMap) {
        inputMap.add("Sequence", EZseq);
        inputMap.add("Conversion", EZtype);
        inputMap.add("Lut", EZlut);
    }

    @Override
    public void declareOutput(final @NotNull VarList outputMap) {
        outputMap.add("Out", varOut);
    }

    @Contract(pure = true)
    public static int getImageType(final @NotNull ColorConversion value) {
        return switch (value) {
            default -> BufferedImage.TYPE_INT_ARGB;
            case RGB -> BufferedImage.TYPE_INT_RGB;
            case GRAY -> BufferedImage.TYPE_BYTE_GRAY;
        };
    }

    public static @NotNull Sequence convertColor(final @NotNull Sequence source, final int imageType, final LUT lut) throws IllegalArgumentException, InterruptedException {
        final Sequence result = new Sequence(OMEUtil.createOMEXMLMetadata(source.getOMEXMLMetadata()));
        // image receiver
        final BufferedImage imgOut = new BufferedImage(source.getSizeX(), source.getSizeY(), imageType);

        result.beginUpdate();
        try {
            for (int t = 0; t < source.getSizeT(); t++)
                for (int z = 0; z < source.getSizeZ(); z++)
                    result.setImage(t, z, IcyBufferedImageUtil.toBufferedImage(source.getImage(t, z), imgOut, lut));

            // rename channels and set final name
            switch (imageType) {
                default:
                case BufferedImage.TYPE_INT_ARGB:
                    result.setChannelName(0, "red");
                    result.setChannelName(1, "green");
                    result.setChannelName(2, "blue");
                    result.setChannelName(3, "alpha");
                    result.setName(source.getName() + " (ARGB rendering)");
                    break;

                case BufferedImage.TYPE_INT_RGB:
                    result.setChannelName(0, "red");
                    result.setChannelName(1, "green");
                    result.setChannelName(2, "blue");
                    result.setName(source.getName() + " (RGB rendering)");
                    break;

                case BufferedImage.TYPE_BYTE_GRAY:
                    result.setChannelName(0, "gray");
                    result.setName(source.getName() + " (gray rendering)");
                    break;
            }
        }
        finally {
            result.endUpdate();
        }

        return result;
    }
}
