/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.infos;

import org.bioimageanalysis.extension.kernel.roi.roi2d.ROI2DRectangle;
import org.bioimageanalysis.icy.extension.plugin.abstract_.Plugin;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.bioimageanalysis.icy.model.roi.ROI;
import org.bioimageanalysis.icy.model.sequence.Sequence;
import org.bioimageanalysis.icy.model.sequence.SequenceUtil;
import org.jetbrains.annotations.NotNull;
import plugins.adufour.blocks.tools.sequence.SequenceBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.VarBoolean;
import plugins.adufour.vars.lang.VarEnum;
import plugins.adufour.vars.lang.VarROIArray;
import plugins.adufour.vars.lang.VarSequence;
import plugins.adufour.vars.util.VarException;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Block to retrieve ROI(s) from a Sequence as 2D Rectangular regions.<br>
 * The <i>origin resolution</i> parameter indicate if we want to rescale ROI to the original image resolution in case we
 * have a Sequence which represents a sub resolution of the original image.
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("Get ROIs as Region")
@IcyPluginIcon(path = "/SequenceBlocks_icon.png")
public class GetRoisAsRegion extends Plugin implements SequenceBlock {
    public enum TypeSelection {
        ALL, SELECTED
    }

    final protected VarSequence inputSequence = new VarSequence("Sequence", null);
    final protected VarEnum<TypeSelection> type = new VarEnum<>("ROI(s) to get", TypeSelection.ALL);
    final protected VarBoolean originResolution = new VarBoolean("Origin resolution", Boolean.TRUE);
    final protected VarROIArray regionsRois = new VarROIArray("Region ROI(s)", null);

    @SuppressWarnings("SwitchStatementWithTooFewBranches")
    @Override
    public void run() {
        final Sequence sequence = inputSequence.getValue();
        if (sequence == null)
            throw new VarException(inputSequence, "Input sequence is null.");

        final List<ROI> rois;
        final List<ROI2DRectangle> result = new ArrayList<>();

        rois = switch (type.getValue()) {
            default -> sequence.getROIs();
            case SELECTED -> sequence.getSelectedROIs();
        };

        final boolean scale = originResolution.getValue();

        // build rectangular ROI(s)
        for (final ROI roi : rois) {
            // get ROI 2D bounds
            Rectangle rect = roi.getBounds5D().toRectangle2D().getBounds();

            // scale if needed
            if (scale)
                rect = SequenceUtil.getOriginRectangle(rect, sequence);

            // add to result
            result.add(new ROI2DRectangle(rect));
        }

        regionsRois.setValue(result.toArray(new ROI[0]));
    }

    @Override
    public void declareInput(final @NotNull VarList inputMap) {
        inputMap.add("sequence", inputSequence);
        inputMap.add("type", type);
        inputMap.add("originResolution", originResolution);
    }

    @Override
    public void declareOutput(final @NotNull VarList outputMap) {
        outputMap.add("regionsRois", regionsRois);
    }
}
