/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.loop;

import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.bioimageanalysis.icy.model.roi.ROI;
import org.bioimageanalysis.icy.model.sequence.Sequence;
import org.bioimageanalysis.icy.model.sequence.SequenceUtil;
import plugins.adufour.blocks.lang.Batch;
import plugins.adufour.blocks.tools.sequence.SequenceBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.Var;
import plugins.adufour.vars.lang.VarROIArray;
import plugins.adufour.vars.lang.VarSequence;
import plugins.adufour.vars.util.VarException;

import java.util.List;

/**
 * Simple batch loop to iterate over a set of Region (ROIs) from an input Sequence.
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("Sequence Region Batch")
@IcyPluginIcon(path = "/SequenceBlocks_icon.png")
public class SequenceRegionBatch extends Batch implements SequenceBlock {
    // important to not initialize here (even with null) and create them in getBatchSource() and getBatchElement()
    protected VarSequence inputSequence;
    protected VarSequence element;
    protected VarROIArray rois;
    protected VarROIArray currentRoi;

    public SequenceRegionBatch() {
        super();
    }

    @Override
    public VarROIArray getBatchSource() {
        if (rois == null)
            rois = new VarROIArray("ROI(s)");

        return rois;
    }

    @Override
    public VarSequence getBatchElement() {
        if (element == null)
            element = new VarSequence("Region Sequence", null);

        return element;
    }

    @Override
    public void declareInput(final VarList inputMap) {
        super.declareInput(inputMap);

        if (inputSequence == null)
            inputSequence = new VarSequence("Sequence", null);

        inputMap.add(inputSequence.getName(), inputSequence);
    }

    @Override
    public void declareOutput(final VarList outputMap) {
        super.declareOutput(outputMap);

        if (currentRoi == null)
            currentRoi = new VarROIArray("Current ROI");

        outputMap.add(currentRoi.getName(), currentRoi);
    }

    @Override
    public void initializeLoop() {
        final Sequence value = inputSequence.getValue();

        if (value == null)
            throw new VarException(inputSequence, "No input sequence indicated");

        if (rois.getValue() == null)
            throw new VarException(rois, "No roi(s) indicated --> no region to iterate over");
    }

    @Override
    public void beforeIteration() {
        // set result in element
        final ROI roi = rois.getValue()[getIterationCounter().getValue()];

        try {
            element.setValue(SequenceUtil.getSubSequence(inputSequence.getValue(), roi));
            currentRoi.setValue(new ROI[]{roi});
        }
        catch (final InterruptedException e) {
            // nothing to do
        }
    }

    @Override
    public boolean isStopConditionReached() {
        return getIterationCounter().getValue() >= rois.getValue().length;
    }

    @Override
    public void declareLoopVariables(final List<Var<?>> loopVariables) {
        super.declareLoopVariables(loopVariables);
        loopVariables.add(inputSequence);
        loopVariables.add(currentRoi);
    }
}
