/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.tprovoost.sequenceblocks.op;

import org.bioimageanalysis.icy.common.type.DataIterator;
import org.bioimageanalysis.icy.extension.plugin.abstract_.Plugin;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.bioimageanalysis.icy.model.image.ImageDataIterator;
import org.bioimageanalysis.icy.model.roi.ROI;
import org.bioimageanalysis.icy.model.sequence.Sequence;
import org.bioimageanalysis.icy.model.sequence.SequenceDataIterator;
import org.jetbrains.annotations.NotNull;
import plugins.adufour.blocks.tools.sequence.SequenceBlock;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.vars.lang.VarROIArray;
import plugins.adufour.vars.lang.VarSequence;
import plugins.adufour.vars.util.VarException;

/**
 * Block to do additive fill of area inside the ROI regions with the specified value.<br>
 * The result of this operation is similar to heatmap production from ROIs.
 *
 * @author Stephane Dallongeville
 */
@IcyPluginName("Additive Fill")
@IcyPluginIcon(path = "/SequenceBlocks_icon.png")
public class AdditiveFillSequence extends Plugin implements SequenceBlock {
    final protected VarSequence inputSequence = new VarSequence("Sequence", null);
    final protected VarROIArray rois = new VarROIArray("Roi(s)");

    @Override
    public void run() {
        final Sequence sequence = inputSequence.getValue();
        if (sequence == null)
            throw new VarException(inputSequence, "Input sequence is null.");

        try {
            if (rois.getValue() != null) {
                for (final ROI roi : rois.getValue())
                    doAdditiveFill(sequence, roi, 1d);

                sequence.dataChanged();
            }
        }
        catch (final InterruptedException e) {
            // nothing to do
        }
    }

    public static void doAdditiveFill(final Sequence sequence, final ROI roi, final double value) throws InterruptedException {
        doAdditiveFill(new SequenceDataIterator(sequence, roi), value);
    }

    public static void doAdditiveFill(final @NotNull DataIterator it, final double value) throws InterruptedException {
        it.reset();

        while (!it.done()) {
            it.set(it.get() + value);
            it.next();
        }

        try {
            // not really nice to do that here, but it's to preserve backward compatibility
            if (it instanceof SequenceDataIterator)
                ((SequenceDataIterator) it).flush();
            else if (it instanceof ImageDataIterator)
                ((ImageDataIterator) it).flush();
        }
        catch (final Throwable t) {
            // ignore
        }
    }

    @Override
    public void declareInput(final @NotNull VarList inputMap) {
        inputMap.add("sequence", inputSequence);
        inputMap.add("rois", rois);
    }

    @Override
    public void declareOutput(final VarList outputMap) {
        //
    }
}